import tape from 'tape';
import {
  is,
  fromJS,
  Iterable,
  List,
  Map,
  OrderedMap
} from 'immutable';

import {
  deepGet,
  deepGetChildren
} from '../get';

import {
  keysToPath,
  keysToPathChildren
} from '../utils';

const nestedMaps = fromJS({
  a: {
    b: "B",
    c: "C",
    d: {
      e: "E"
    }
  },
  f: "F"
});

const nestedLists = fromJS([
  ["A","B","C"],
  ["D",["E","F"]],
  "G"
]);

const childLists = fromJS({
  name: "root",
  children: [
    {
      name: "child 1",
      children: [
        {name: "grandchild 1"},
        {name: "grandchild 2"}
      ]
    },
    {
      name: "child 2",
      children: []
    },
    {name: "child 3"}
  ]
});

tape('deepGet', test => {
    test.plan(5);

    test.ok(is(
      nestedMaps.update(deepGet()),
      nestedMaps
    ), 'no params returns clone of original tree');

    test.ok(is(
    	nestedMaps.update(deepGet(['a','b'])),
    	"B"
    ), 'using no childPath works like getIn');

     test.ok(is(
      nestedMaps.update(deepGet(['a','b','c','d'], null, "not set")),
      "not set"
    ), 'not set value is returned if node at keys does not exist');

    test.ok(is(
      childLists.update(deepGet(null, ['children'])).get('name'),
      "root"
    ), 'no keys with childPath returns root node');

    test.ok(is(
      childLists.update(deepGet([0,1], ['children'])),
      fromJS({name: "grandchild 2"})
    ), 'using childPath works correctly');
});

tape('deepGetChildren', test => {
    test.plan(5);

    test.ok(is(
      nestedMaps.update(deepGetChildren()),
      nestedMaps
    ), 'no params returns clone of original tree');

    test.ok(is(
      nestedMaps.update(deepGetChildren(['a','b'])),
      "B"
    ), 'using no childPath works like getIn / deepget');

     test.ok(is(
      nestedMaps.update(deepGetChildren(['a','b','c'], null, "not set")),
      "not set"
    ), 'not set value is returned if no children at node');

    test.ok(is(
      childLists.update(deepGetChildren(null, ['children'])),
      childLists.get('children')
    ), 'no keys with childPath returns root node children');

    test.ok(is(
      childLists.update(deepGetChildren([0], ['children'])),
      childLists.getIn(['children', 0, 'children'])
    ), 'using childPath works correctly');
});