import tape from 'tape';
import {
  is,
  fromJS,
  Iterable,
  List,
  Map,
  OrderedMap
} from 'immutable';

import {
  deepUpdate,
  deepUpdateChildren
} from '../update';


const nestedMaps = fromJS({
  a: {
    b: "B"
  },
  f: "F"
});

const childLists = fromJS({
  name: "root",
  children: [
    {
      name: "child 1",
      children: [
        {name: "grandchild 1"},
        {name: "grandchild 2"}
      ]
    }
  ]
});

tape('deepUpdate', test => {
    test.plan(4);

    test.ok(is(
    	nestedMaps.update(deepUpdate(['a'], node => node.set('abc', 123))),
    	fromJS({
		  a: {
		    b: "B",
		    abc: 123
		  },
		  f: "F"
		})
    ), 'using no childPath works like updateIn');

    test.ok(is(
    	nestedMaps.update(deepUpdate(['g'], node => node.set('abc', 123), null, fromJS({def: 456}))),
    	fromJS({
		  a: {
		    b: "B"
		  },
		  f: "F",
		  g: {
		  	def: 456,
		  	abc: 123
		  }
		})
    ), 'can use notSetValue with no childPath');

    test.ok(is(
      childLists.update(deepUpdate([0,1], node => node.set('abc', 123), ['children'])),
      fromJS({
	    name: "root",
	    children: [
	      {
	        name: "child 1",
	        children: [
	          {name: "grandchild 1"},
	          {name: "grandchild 2", abc:123}
	        ]
	      }
	    ]
	  })
    ), 'using childPath works correctly');

    test.ok(is(
      childLists.update(deepUpdate([0,2], node => node.set('abc', 123), ['children'], fromJS({def: 456}))),
      fromJS({
	    name: "root",
	    children: [
	      {
	        name: "child 1",
	        children: [
	          {name: "grandchild 1"},
	          {name: "grandchild 2"},
	          {abc: 123, def: 456}
	        ]
	      }
	    ]
	  })
    ), 'can use notSetValue with childPath');
});

tape('deepUpdateChildren', test => {
    test.plan(3);

    test.ok(is(
    	nestedMaps.update(deepUpdateChildren(['a'], kids => kids.set('c','C'))),
    	fromJS({
		  a: {
		    b: "B",
		    c: "C"
		  },
		  f: "F"
		})
    ), 'using no childPath works like updateIn');

    test.ok(is(
    	nestedMaps.update(deepUpdateChildren(['a','b'], kids => kids.set('c','C'))),
    	nestedMaps
    ), 'deepUpdateChildren with no childPath on a non iterable node means no change is made');

    test.ok(is(
      childLists.update(deepUpdateChildren([0,0], kids => kids.push(Map({'abc': 123})), ['children'], fromJS([{def: 456}]))),
      fromJS({
	    name: "root",
	    children: [
	      {
	        name: "child 1",
	        children: [
	          {
	          	name: "grandchild 1",
	          	children: [
	          	  {def: 456},
	          	  {abc: 123}
	          	]
	      	  },
	          {name: "grandchild 2"}
	        ]
	      }
	    ]
	  })
    ), 'can update with notSetValue and childPath');
});